import 'package:json_annotation/json_annotation.dart';

part 'notification.g.dart';

@JsonSerializable()
class NotificationList {
  @JsonKey(name: "TodoList")
  List<Activity> todoList;

  @JsonKey(name: "OutStandingList")
  List<Outstanding> outstandingList;

  NotificationList();

  factory NotificationList.fromJson(Map<String, dynamic> json) =>
      _$NotificationListFromJson(json);
  Map<String, dynamic> toJson() => _$NotificationListToJson(this);
}

@JsonSerializable()
class Activity {
  @JsonKey(name: "Id")
  int id;

  @JsonKey(name: "IdToken")
  int idToken;

  @JsonKey(name: "IdStaff")
  int idStaff;

  @JsonKey(name: "NotificationDate")
  DateTime notificationDate;

  @JsonKey(name: "IdProcessInstance")
  int idProcessInstance;

  @JsonKey(name: "ProcessInstanceName")
  String processInstanceName;

  @JsonKey(name: "ActivityName")
  String activityName;

  @JsonKey(name: "IsViewed")
  bool isViewed;

  @JsonKey(name: "Interval")
  Interval interval;

  @JsonKey(name: "ProcessCreatedById")
  int processCreatedById;

  @JsonKey(name: "ProcessCreatedByName")
  String processCreatedByName;

  @JsonKey(name: "IsAutoCoachClaim")
  bool isAutoCoachClaim;

  @JsonKey(name: "IdCurrentFlow")
  dynamic idCurrentFlow;

  @JsonKey(name: "CalledByProcess")
  String calledByProcess;

  @JsonKey(name: "IdProcessSnapshot")
  int idProcessSnapshot;

  @JsonKey(name: "spreadsheetLastUpdate")
  DateTime spreadsheetLastUpdate;

  @JsonKey(name: "NotificationInfo")
  String notificationInfo;

  @JsonKey(name: "NotificationInfoResult")
  String notificationInfoResult;

  @JsonKey(name: "ProcessDateCreated")
  DateTime processDateCreated;

  @JsonKey(name: "ProcessCreatedDuration")
  double processCreatedDuration;

  @JsonKey(name: "ProcessName")
  String processName;

  @JsonKey(name: "SLATime")
  int slaTime;

  Activity();

  factory Activity.fromJson(Map<String, dynamic> json) =>
      _$ActivityFromJson(json);
  Map<String, dynamic> toJson() => _$ActivityToJson(this);

  bool isMatch(String filter) {
    filter = filter.toLowerCase();
    return processName.toLowerCase().contains(filter) ||
        activityName.toLowerCase().contains(filter) ||
        (notificationInfo ?? "").toLowerCase().contains(filter) ||
        (processCreatedByName ?? "").toLowerCase().contains(filter) ||
        idProcessInstance.toString() == filter;
  }
}

@JsonSerializable()
class Outstanding extends Activity {
  @JsonKey(name: "DateCreated")
  DateTime dateCreated;
  @JsonKey(name: "OpenByStaffList")
  dynamic openByStaffList;
  @JsonKey(name: "ProcessInstanceName")
  String processInstanceName;
  @JsonKey(name: "MasterProcessName")
  String masterProcessName;

  Outstanding();

  factory Outstanding.fromJson(Map<String, dynamic> json) =>
      _$OutstandingFromJson(json);
  Map<String, dynamic> toJson() => _$OutstandingToJson(this);
}

@JsonSerializable()
class Interval {
  @JsonKey(name: "IsUsed")
  bool isUsed;
  @JsonKey(name: "Days")
  int days;
  @JsonKey(name: "Hours")
  int hours;
  @JsonKey(name: "Minutes")
  int minutes;
  @JsonKey(name: "TotalMinutes")
  int totalMinutes;

  Interval({
    this.isUsed,
    this.days,
    this.hours,
    this.minutes,
    this.totalMinutes,
  });

  factory Interval.fromJson(Map<String, dynamic> json) =>
      _$IntervalFromJson(json);
  Map<String, dynamic> toJson() => _$IntervalToJson(this);
}
