import 'package:json_annotation/json_annotation.dart';
import 'package:xml/xml.dart';

part 'variable.g.dart';

@JsonSerializable()
class BusinessData {
  @JsonKey(name: "PublicVariables")
  List<Variable> publicVariables;

  @JsonKey(name: "PrivateVariables")
  List<Variable> privateVariables;

  @JsonKey(name: "Value")
  List<VariableValue> values;

  BusinessData();

  factory BusinessData.fromJson(Map<String, dynamic> json) =>
      _$BusinessDataFromJson(json);
  Map<String, dynamic> toJson() => _$BusinessDataToJson(this);

  VariableValue valueOf(String variableId) {
    for (var value in values) {
      if (value.id == variableId) {
        return value;
      }
    }

    return null;
  }

  Variable findByName(String name) {
    for (var variable in privateVariables) {
      if (variable.name == name) return variable;
    }

    for (var variable in publicVariables) {
      if (variable.name == name) return variable;
    }

    return null;
  }
}

@JsonSerializable()
class Variable {
  @JsonKey(name: "Id")
  String id;

  @JsonKey(name: "Data")
  String xmlData;

  Variable();

  XmlElement _document;
  String get name => _document.getAttribute("name");
  String get type => _document.getAttribute("type");
  int get dataSourceType =>
      int.tryParse(_document.getAttribute("dataSourceType"));
  int get valueType => int.tryParse(_document.getAttribute("valueType"));

  init() {
    try {
      final doc = XmlDocument.parse(xmlData.replaceAll("]20]20>", "]]>"));
      _document = doc.getElement("privateVariable");
    } catch (e) {
      print(e);
    }
  }

  factory Variable.fromJson(Map<String, dynamic> json) {
    return _$VariableFromJson(json)..init();
  }

  Map<String, dynamic> toJson() => _$VariableToJson(this);
}

@JsonSerializable()
class VariableValue {
  @JsonKey(name: "Id")
  String id;

  @JsonKey(name: "Type")
  String type;

  @JsonKey(name: "ListDefaultControlType")
  int listDefaultControlType;

  @JsonKey(name: "IsList")
  bool isList;

  @JsonKey(name: "RawValue")
  String rawValue;

  VariableValue();

  factory VariableValue.fromJson(Map<String, dynamic> json) =>
      _$VariableValueFromJson(json);
  Map<String, dynamic> toJson() => _$VariableValueToJson(this);

  String asString() {
    return rawValue;
  }

  int asInt() {
    return int.tryParse(rawValue);
  }

  double asDouble() {
    return double.tryParse(rawValue);
  }
}
