import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/pages/process_activity_list.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/widgets/bpmn_diagram.dart';

class DiagramPage extends StatelessWidget {
  const DiagramPage({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    bool isLarge = MediaQuery.of(context).size.width > 600;
    return Scaffold(
      appBar: AppBar(
        title: Text("Diagram"),
      ),
      body: isLarge
          ? Row(
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: [
                Expanded(
                  child: ClipRect(child: buildDiagram()),
                ),
                VerticalDivider(
                  width: 1.0,
                ),
                SizedBox(
                  width: 350,
                  child: buildActivityList(),
                )
              ],
            )
          : Stack(
              clipBehavior: Clip.antiAlias,
              children: [
                Positioned.fill(
                  child: buildDiagram(),
                ),
                DraggableScrollableSheet(
                  builder: (context, controller) => Material(
                    color: Colors.white,
                    type: MaterialType.canvas,
                    child: buildActivityList(controller: controller),
                  ),
                )
              ],
            ),
    );
  }

  Widget buildActivityList({ScrollController controller}) {
    return SingleChildScrollView(
      controller: controller,
      child: ProcessActivityList(),
    );
  }

  Widget buildDiagram() {
    return Consumer<ProcessProvider>(
      builder: (context, provider, child) {
        return BpmnDiagram(
          directForm: provider.directForm,
          tokens: provider.tokens,
        );
      },
    );
  }
}
