import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/pages/notification_page.dart';
import 'package:retgoo_sdk/pages/shortcut_page.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';

class HomePage extends StatefulWidget {
  HomePage({Key key}) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  int _currentIndex = 0;
  bool get isLarge => MediaQuery.of(context).size.width > 600;

  @override
  Widget build(BuildContext context) {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.light.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: Scaffold(
        backgroundColor: isLarge ? null : Theme.of(context).primaryColor,
        body: isLarge ? buildDesktop() : buildMobile(),
        bottomNavigationBar: isLarge
            ? null
            : BottomNavigationBar(
                backgroundColor: Colors.white,
                elevation: 16.0,
                items: [
                  BottomNavigationBarItem(
                    icon: Icon(CupertinoIcons.app),
                    label: "Processes",
                  ),
                  BottomNavigationBarItem(
                    icon: Icon(CupertinoIcons.bell),
                    label: "Tasks",
                  ),
                  BottomNavigationBarItem(
                    icon: Icon(CupertinoIcons.settings),
                    label: "Profile",
                  ),
                ],
                currentIndex: _currentIndex,
                onTap: (index) {
                  setState(() {
                    _currentIndex = index;
                  });
                },
              ),
      ),
    );
  }

  buildDesktop() {
    return Row(
      children: [
        NavigationRail(
          elevation: 8.0,
          leading: CircleAvatar(
            child: Icon(CupertinoIcons.person),
          ),
          destinations: [
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.app),
              label: Text("Processes"),
            ),
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.bell),
              label: Text("Tasks"),
            ),
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.settings),
              label: Text("Profile"),
            ),
          ],
          selectedIndex: _currentIndex,
          onDestinationSelected: (index) {
            setState(() {
              _currentIndex = index;
            });
          },
        ),
        Expanded(
          child: IndexedStack(
            index: _currentIndex,
            children: [
              ShortcutPage(),
              NotificationPage(),
              Container(),
            ],
          ),
        ),
      ],
    );
  }

  buildMobile() {
    return SafeArea(
      bottom: false,
      child: Selector<AuthenticationProvider, User>(
        selector: (context, provider) => provider.currentUser,
        builder: (context, user, child) {
          return NestedScrollView(
            headerSliverBuilder: (context, scrolling) => [
              SliverAppBar(
                expandedHeight: 250,
                flexibleSpace: FlexibleSpaceBar(
                  background: Container(
                    alignment: Alignment.center,
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        CircleAvatar(
                          backgroundImage: MemoryImage(user.imageBytes),
                          radius: 32,
                        ),
                        Text(
                          user.firstName,
                          style: TextStyle(
                            fontSize: 24.0,
                            fontWeight: FontWeight.w900,
                            color: Colors.white,
                          ),
                        ),
                        const SizedBox(
                          height: 16.0,
                        ),
                        OutlineButton(
                          child: Text("LOGOUT"),
                          textColor: Colors.white,
                          shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(100.0),
                          ),
                          borderSide: BorderSide(
                            color: Colors.white,
                          ),
                          onPressed: () {
                            Provider.of<AuthenticationProvider>(context,
                                    listen: false)
                                .logout();
                          },
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            ],
            body: Container(
              margin: const EdgeInsets.only(top: 8.0),
              decoration: BoxDecoration(
                color: Theme.of(context).scaffoldBackgroundColor,
                borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(24.0),
                  topRight: Radius.circular(24.0),
                ),
                boxShadow: [
                  BoxShadow(
                    offset: Offset(0, -1),
                    blurRadius: 8.0,
                    color: Colors.black.withOpacity(0.125),
                  ),
                ],
              ),
              child: ClipRRect(
                borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(24.0),
                  topRight: Radius.circular(24.0),
                ),
                child: IndexedStack(
                  index: _currentIndex,
                  children: [
                    ShortcutPage(),
                    NotificationPage(),
                    Container(),
                  ],
                ),
              ),
            ),
          );
        },
      ),
    );
  }
}
