import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:md2_tab_indicator/md2_tab_indicator.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/notification.dart';
import 'package:retgoo_sdk/pages/process_page.dart';
import 'package:retgoo_sdk/providers/notification_provider.dart';

class NotificationPage extends StatelessWidget {
  const NotificationPage({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Selector<NotificationProvider, NotificationList>(
      selector: (context, provider) => provider.list,
      builder: (context, list, child) {
        if (list == null) {
          return Center(
            child: CircularProgressIndicator(),
          );
        }

        return DefaultTabController(
          length: 2,
          child: Column(
            children: [
              const SizedBox(
                height: 16.0,
              ),
              Padding(
                padding: const EdgeInsets.symmetric(
                  vertical: 8.0,
                  horizontal: 16.0,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: [
                    Text(
                      "Tasks",
                      style: TextStyle(
                        fontSize: 32.0,
                        fontWeight: FontWeight.w900,
                      ),
                    ),
                    Text(
                      "Jobs to be done",
                      style: TextStyle(),
                    ),
                  ],
                ),
              ),
              TabBar(
                indicator: MD2Indicator(
                  indicatorHeight: 3,
                  indicatorColor: Color(0xff1a73e8),
                  indicatorSize: MD2IndicatorSize.normal,
                ),
                labelColor: Colors.indigo,
                tabs: [
                  Tab(
                    text: "Todo",
                  ),
                  Tab(
                    text: "Oustanding",
                  ),
                ],
              ),
              Divider(
                height: 1,
              ),
              Expanded(
                child: TabBarView(
                  children: [
                    ListView.builder(
                      padding: const EdgeInsets.all(16.0),
                      itemCount: list.todoList?.length ?? 0,
                      itemBuilder: (context, index) {
                        final item = list.todoList.elementAt(index);
                        return Card(
                          child: ListTile(
                            isThreeLine: true,
                            title: Text(
                              item.processName,
                              style: TextStyle(
                                fontWeight: FontWeight.w900,
                              ),
                            ),
                            leading: Icon(
                              CupertinoIcons.timer_fill,
                              size: 32.0,
                              color: Colors.deepPurple,
                            ),
                            trailing: Icon(CupertinoIcons.chevron_right_circle),
                            subtitle: Column(
                              crossAxisAlignment: CrossAxisAlignment.stretch,
                              children: [
                                Text(item.activityName),
                                Text(item.processCreatedByName),
                              ],
                            ),
                            onTap: () {
                              Navigator.push(
                                context,
                                MaterialPageRoute(
                                  builder: (context) => ProcessPage(
                                    idProcessInstance: item.idProcessInstance,
                                    idActivityNotification: item.id,
                                  ),
                                ),
                              );
                            },
                          ),
                        );
                      },
                    ),
                    ListView.builder(
                      itemCount: list.outstandingList?.length ?? 0,
                      itemBuilder: (context, index) {
                        final item = list.outstandingList.elementAt(index);
                        return ListTile(
                          title: Text(item.masterProcessName),
                        );
                      },
                    ),
                  ],
                ),
              ),
            ],
          ),
        );
      },
    );
  }
}
