import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/process_activity.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/widgets/custom/stopwatch_text.dart';

class ProcessActivityList extends StatelessWidget {
  const ProcessActivityList({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Consumer<ProcessProvider>(
      builder: (context, notifier, child) {
        if (notifier?.activities == null || notifier.activities.isEmpty) {
          return Padding(
            padding: const EdgeInsets.all(16.0),
            child: Center(
              child: CircularProgressIndicator(),
            ),
          );
        }

        return Column(
          children: notifier.activities
              .map<Widget>(
                (item) => buildActivityItem(
                  context,
                  item,
                  notifier.activities.indexOf(item),
                  notifier.activities,
                ),
              )
              .toList(),
        );
      },
    );
  }

  Widget buildActivityItem(
    BuildContext context,
    ProcessActivity item,
    int index,
    List<ProcessActivity> list,
  ) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 16.0),
      height: 200,
      child: Row(
        children: <Widget>[
          buildNomor(context, index, item, list),
          const SizedBox(
            width: 8.0,
          ),
          buildCard(item, context),
        ],
      ),
    );
  }

  Widget buildCard(ProcessActivity item, BuildContext context) {
    return Expanded(
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Card(
          child: Padding(
            padding: const EdgeInsets.all(10.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: <Widget>[
                Text(
                  item.laneName ?? "",
                  style: TextStyle(
                    fontSize: 16.0,
                    fontWeight: FontWeight.bold,
                  ),
                ),
                Text(item.parentActivityName ?? ""),
                Container(
                  margin: const EdgeInsets.symmetric(vertical: 8.0),
                  height: 1,
                  color: Theme.of(context).dividerColor,
                ),
                item.isCompleted
                    ? buildCompletedInfo(item, context)
                    : buildWaitingInfo(context, item),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget buildWaitingInfo(BuildContext context, ProcessActivity item) {
    return Expanded(
      child: Container(
        alignment: Alignment.center,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text(
              "Waiting for",
              style: TextStyle(
                fontSize: 12.0,
              ),
            ),
            StopwatchText(
              key: ObjectKey(item),
              now: DateTime.now(),
              start: item.dateCreated,
              style: TextStyle(
                fontSize: 24.0,
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget buildCompletedInfo(ProcessActivity item, BuildContext context) {
    return Expanded(
      child: Container(
        alignment: Alignment.center,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            Text.rich(
              TextSpan(
                children: [
                  TextSpan(
                    text: DateFormat("EEEE, dd MMMM yyyy HH:mm")
                        .format(item.dateCompleted.toLocal())
                        .toUpperCase(),
                  ),
                ],
              ),
              style: TextStyle(
                fontSize: 12.0,
              ),
            ),
            const SizedBox(
              height: 8.0,
            ),
            // Icon(
            //   PortalType.assetPath(item.portalType),
            //   size: 24.0,
            // ),
            const SizedBox(
              height: 8.0,
            ),
            Text.rich(
              TextSpan(
                children: [
                  TextSpan(
                    text: item.staffName.toUpperCase(),
                    style: TextStyle(
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ],
              ),
              style: TextStyle(
                fontSize: 12.0,
              ),
            ),
            Text.rich(
              TextSpan(
                children: [
                  TextSpan(
                    text: item.deviceName ?? "Desktop",
                    style: TextStyle(
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ],
              ),
              style: TextStyle(
                fontSize: 12.0,
              ),
            ),
          ],
        ),
      ),
    );
  }

  Widget buildNomor(BuildContext context, int index, ProcessActivity item,
      List<ProcessActivity> list) {
    return SizedBox(
      width: 48,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: Container(
              width: index > 0 ? 2 : 0,
              color: Theme.of(context).dividerColor,
            ),
          ),
          const SizedBox(
            height: 4.0,
          ),
          Material(
            elevation: 4.0,
            type: MaterialType.circle,
            color: item.isCompleted ? Colors.green : Colors.purple,
            child: Container(
              constraints: BoxConstraints(
                minWidth: 36,
                minHeight: 36,
              ),
              alignment: Alignment.center,
              child: Text(
                "${index + 1}",
                style: TextStyle(
                  color: Colors.white,
                ),
              ),
            ),
          ),
          const SizedBox(
            height: 4.0,
          ),
          Expanded(
            child: Container(
              width: index < list.length - 1 ? 2 : 0,
              color: Theme.of(context).dividerColor,
            ),
          ),
        ],
      ),
    );
  }
}
