import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/shortcut.dart';
import 'package:retgoo_sdk/providers/shortcut_provider.dart';

class ShortcutPage extends StatefulWidget {
  const ShortcutPage({Key key}) : super(key: key);

  @override
  _ShortcutPageState createState() => _ShortcutPageState();
}

class _ShortcutPageState extends State<ShortcutPage> {
  String filter = "";

  bool get isLarge => MediaQuery.of(context).size.width > 600;

  @override
  Widget build(BuildContext context) {
    return Consumer<ShortcutProvider>(
      builder: (context, provider, child) {
        var gs = provider.groups;
        if (gs == null) {
          return Center(
            child: CircularProgressIndicator(),
          );
        }

        Map<String, Map<String, Tile>> groups = <String, Map<String, Tile>>{};
        for (var group in gs.keys) {
          final entries = gs[group];
          final vals = <String, Tile>{};
          for (var id in entries.keys) {
            final tile = entries[id];
            if (tile.caption.toLowerCase().contains(filter.toLowerCase())) {
              vals[id] = tile;
            }
          }

          if (vals.isNotEmpty) {
            groups[group] = vals;
          }
        }

        return Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            const SizedBox(
              height: 16.0,
            ),
            if (!isLarge) ...[
              Padding(
                padding: const EdgeInsets.symmetric(
                  vertical: 8.0,
                  horizontal: 16.0,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: [
                    Text(
                      "Shortcuts",
                      style: TextStyle(
                        fontSize: 32.0,
                        fontWeight: FontWeight.w900,
                      ),
                    ),
                    Text(
                      "Start your business process",
                      style: TextStyle(),
                    ),
                  ],
                ),
              ),
              Padding(
                padding: const EdgeInsets.symmetric(
                  vertical: 0.0,
                  horizontal: 16.0,
                ),
                child: Card(
                  margin: const EdgeInsets.all(0.0),
                  child: TextField(
                    onChanged: (value) {
                      setState(() {
                        filter = value;
                      });
                    },
                    decoration: InputDecoration(
                      border: OutlineInputBorder(borderSide: BorderSide.none),
                      contentPadding: const EdgeInsets.symmetric(
                        horizontal: 16.0,
                        vertical: 10.0,
                      ),
                      prefixIcon: Icon(Icons.search),
                      hintText: "type to find",
                    ),
                  ),
                ),
              ),
            ] else
              Padding(
                padding: const EdgeInsets.symmetric(
                  horizontal: 16.0,
                  vertical: 8.0,
                ),
                child: Row(
                  children: [
                    Expanded(
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            "Shortcuts",
                            style: TextStyle(
                              fontSize: 32.0,
                              fontWeight: FontWeight.w900,
                            ),
                          ),
                          Text(
                            "Start your business process",
                            style: TextStyle(),
                          ),
                        ],
                      ),
                    ),
                    const SizedBox(
                      width: 24.0,
                    ),
                    ConstrainedBox(
                      constraints: BoxConstraints(
                        maxWidth: 350,
                      ),
                      child: Card(
                        margin: const EdgeInsets.all(0.0),
                        child: TextField(
                          onChanged: (value) {
                            setState(() {
                              filter = value;
                            });
                          },
                          decoration: InputDecoration(
                            border:
                                OutlineInputBorder(borderSide: BorderSide.none),
                            contentPadding: const EdgeInsets.symmetric(
                              horizontal: 16.0,
                              vertical: 10.0,
                            ),
                            prefixIcon: Icon(Icons.search),
                            hintText: "type to find",
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            Expanded(
              child: Scrollbar(
                child: ListView.separated(
                  scrollDirection: isLarge ? Axis.horizontal : Axis.vertical,
                  padding: const EdgeInsets.all(16.0),
                  itemCount: groups.length,
                  separatorBuilder: (context, index) => isLarge
                      ? const SizedBox(width: 64.0)
                      : const SizedBox(height: 16.0),
                  itemBuilder: (context, index) {
                    final group = groups.keys.elementAt(index);
                    return buildGroup(group, groups, context);
                  },
                ),
              ),
            ),
          ],
        );
      },
    );
  }

  Widget buildGroup(String group, Map<String, Map<String, Tile>> groups,
      BuildContext context) {
    return Column(
      crossAxisAlignment:
          isLarge ? CrossAxisAlignment.start : CrossAxisAlignment.stretch,
      children: [
        Text(
          group,
          style: TextStyle(
            fontSize: 16.0,
          ),
        ),
        const SizedBox(
          height: 8.0,
        ),
        if (isLarge)
          Expanded(
            child: GridView.builder(
              scrollDirection: Axis.horizontal,
              shrinkWrap: true,
              physics: NeverScrollableScrollPhysics(),
              gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
                maxCrossAxisExtent: 250,
                crossAxisSpacing: 8.0,
                mainAxisSpacing: 8.0,
                childAspectRatio: 1.3,
              ),
              itemCount: groups[group].values.length,
              itemBuilder: (context, index) {
                final tile = groups[group].values.elementAt(index);
                return Column(
                  children: [
                    Card(
                      elevation: 0.0,
                      color: Colors.green,
                      child: AspectRatio(
                        aspectRatio: 1.0,
                        child: InkWell(
                          onTap: () {
                            Provider.of<ShortcutProvider>(context,
                                    listen: false)
                                .createProcess(tile, "");
                          },
                          child: Container(
                            alignment: Alignment.center,
                            child: tile.imageBase64 != null
                                ? Image.memory(
                                    tile.imageBase64,
                                    color: Colors.white,
                                    width: 48.0,
                                    height: 48.0,
                                  )
                                : null,
                          ),
                        ),
                      ),
                    ),
                    Spacer(),
                    Text(
                      tile.caption,
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis,
                      softWrap: false,
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                    Spacer(),
                    Spacer(),
                  ],
                );
              },
            ),
          )
        else
          for (var tile in groups[group].values)
            Card(
              child: InkWell(
                onTap: () {
                  Provider.of<ShortcutProvider>(context, listen: false)
                      .createProcess(tile, "");
                },
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Row(
                    children: [
                      Container(
                        height: 48,
                        width: 48,
                        alignment: Alignment.center,
                        decoration: BoxDecoration(
                          color: Colors.green,
                          borderRadius: BorderRadius.circular(8.0),
                        ),
                        child: tile.imageBase64 != null
                            ? Image.memory(
                                tile.imageBase64,
                                color: Colors.white,
                                width: 24.0,
                                height: 24.0,
                              )
                            : Container(
                                width: 24,
                                height: 24,
                              ),
                      ),
                      const SizedBox(
                        width: 8.0,
                      ),
                      Expanded(
                        child: Column(
                          crossAxisAlignment: CrossAxisAlignment.stretch,
                          children: <Widget>[
                            Text(
                              tile.caption,
                              style: TextStyle(
                                fontWeight: FontWeight.bold,
                              ),
                            ),
                            Text(
                              tile.description.isEmpty
                                  ? "tidak ada keterangan"
                                  : tile.description,
                              style: TextStyle(
                                color: Colors.grey,
                                fontSize: 12.0,
                              ),
                            ),
                          ],
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            ),
        const SizedBox(
          height: 16.0,
        ),
      ],
    );
  }
}
