import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/protocol/api.dart';
import 'package:shared_preferences/shared_preferences.dart';

enum AuthenticationState {
  initializing,
  authenticated,
  unautenticated,
}

class AuthenticationProvider with ChangeNotifier {
  AuthenticationState _state = AuthenticationState.initializing;
  AuthenticationState get state => _state;

  User _user;
  User get currentUser => _user;

  AuthenticationProvider() {
    SharedPreferences.getInstance().then((value) async {
      if (value.containsKey("token")) {
        accessToken = value.getString("token");
        _state = AuthenticationState.authenticated;
        await refreshUser();
      } else {
        _state = AuthenticationState.unautenticated;
      }

      notifyListeners();
    });
  }

  Future<bool> login({String username, String password}) async {
    final result = await api.post("user/authentication", data: {
      "username": username,
      "password": password,
    });

    if (result.statusCode == 200) {
      if (result.data["success"]) {
        accessToken = result.data["token"];
        final sp = await SharedPreferences.getInstance();
        await sp.setString("token", accessToken);

        _state = AuthenticationState.authenticated;
        await refreshUser();
        notifyListeners();
        return true;
      }
    }

    return false;
  }

  Future<void> logout() async {
    final sp = await SharedPreferences.getInstance();
    await sp.remove("token");
    sp.reload();
    accessToken = null;
    _state = AuthenticationState.unautenticated;
    notifyListeners();
  }

  Future<void> refreshUser() async {
    final response = await api.get("user/GetProfile");
    if (response.statusCode == 200) {
      _user = User.fromJson(response.data);

      try {
        final bytes = base64.decode(_user.image48);
        _user.imageBytes = bytes;
      } catch (e) {}

      notifyListeners();
    }
  }
}
