import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:retgoo_sdk/models/shortcut.dart';
import 'package:retgoo_sdk/protocol/api.dart';
import 'package:xml/xml.dart' as xml;

class ShortcutProvider with ChangeNotifier {
  Map<String, Map<String, Tile>> _tileGroups;
  Map<String, Map<String, Tile>> get groups => _tileGroups;

  ShortcutProvider() {
    refresh();
  }

  static decodeResponse(List response) async {
    Map<String, Map<String, Tile>> result;
    if (response != null) {
      result = {};
      for (var item in response) {
        var xmlDoc = item["xml_doc"];
        var xmlDocument = xml.XmlDocument.parse(xmlDoc);

        String customMenu = item["custom_menu"] ?? "";
        var groups = xmlDocument.findAllElements("TileGroup").toList();
        for (var group in groups) {
          Map<String, Tile> tileGroup;
          String groupCaption = group.getAttribute("caption");

          if (result.containsKey(groupCaption)) {
            tileGroup = result[groupCaption];
          } else {
            tileGroup = {};
            result[groupCaption] = tileGroup;
          }

          var tiles = group.findAllElements("Tile").toList();
          for (var tile in tiles) {
            if (customMenu.isNotEmpty) {
              if (!customMenu.contains(tile.getAttribute("name"))) {
                continue;
              }
            }

            Uint8List imageBase64;
            try {
              xml.XmlElement imageEl = tile.findElements("Image").first;
              if (imageEl != null) {
                String data = imageEl.text;
                if (data != null && data.isNotEmpty) {
                  imageBase64 = base64.decode(data);
                }
              }
            } catch (_) {}

            Tile tileItem = Tile(
              caption: tile.getAttribute("caption"),
              description: tile.getAttribute("description"),
              name: tile.getAttribute("name"),
              idMasterProcess: int.parse(tile
                  .findElements("TileObjectTarget")
                  .first
                  .getAttribute("idMasterProcess")),
              itemSize: int.parse(
                tile.findElements("ItemSize").first.text,
              ),
              imageBase64: imageBase64,
            );

            tileGroup[tileItem.name] = tileItem;
          }
        }
      }
    }

    return result;
  }

  Future<void> refresh() async {
    final response = await api.get("shortcuts/all");
    if (response.statusCode == 200) {
      final data = await decodeResponse(response.data);
      _tileGroups = data;
      notifyListeners();
    }
  }

  Future<int> createProcess(Tile tile, String note) async {
    final result = await api.post("process/createprocess", data: {
      "id_master_process": tile.idMasterProcess,
      "portal_type": 11,
      "note": note,
    });
    if (result.statusCode == 200) {
      return result.data["id_process_instance"];
    }

    return 0;
  }
}
