import 'dart:async';

import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class StopwatchText extends StatefulWidget {
  final DateTime start;
  final DateTime now;
  final TextStyle style;

  StopwatchText({
    Key key,
    @required this.start,
    this.now,
    this.style,
  }) : super(key: key);

  @override
  _StopwatchTextState createState() => _StopwatchTextState();
}

class _StopwatchTextState extends State<StopwatchText> {
  Duration _offset = Duration(milliseconds: 0);
  Duration duration = Duration(milliseconds: 0);
  Timer _timer;

  @override
  void initState() {
    super.initState();
    if (widget.now != null) {
      _offset = DateTime.now().difference(widget.now);
    } else {
      _offset = Duration(
        milliseconds: 0,
      );
    }

    _timer = Timer.periodic(Duration(seconds: 1), updateText);
    updateText(null);
  }

  updateText(_) {
    setState(
      () {
        final now = DateTime.now().toUtc();
        final start = widget.start;
        duration = now.add(_offset).difference(start);
      },
    );
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    int second = duration.inSeconds.remainder(60);
    int minutes = duration.inMinutes.remainder(60);
    int hours = duration.inHours.remainder(24);
    int days = duration.inDays;

    return Text.rich(
      TextSpan(
        children: <InlineSpan>[
          if (days > 0)
            TextSpan(
              text: NumberFormat("00").format(days),
              style: TextStyle(
                fontWeight: FontWeight.bold,
              ),
            ),
          if (days > 0)
            TextSpan(
              text: " d",
              style: TextStyle(
                fontWeight: FontWeight.w300,
                fontSize: 12.0,
              ),
            ),
          TextSpan(
            text: NumberFormat("00").format(hours),
            style: TextStyle(
              fontWeight: FontWeight.bold,
            ),
          ),
          TextSpan(
            text: " h ",
            style: TextStyle(
              fontWeight: FontWeight.w300,
              fontSize: 12.0,
            ),
          ),
          TextSpan(
            text: NumberFormat("00").format(minutes),
            style: TextStyle(
              fontWeight: FontWeight.bold,
            ),
          ),
          TextSpan(
            text: "m ",
            style: TextStyle(
              fontWeight: FontWeight.w300,
              fontSize: 12.0,
            ),
          ),
          TextSpan(
            text: NumberFormat("00").format(second),
            style: TextStyle(
              fontWeight: FontWeight.bold,
            ),
          ),
          TextSpan(
            text: " s ",
            style: TextStyle(
              fontWeight: FontWeight.w300,
              fontSize: 12.0,
            ),
          ),
        ],
      ),
      style: widget.style ?? TextStyle(),
    );
  }
}
