import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/forms/forms.dart';
import 'package:retgoo_sdk/pages/home_page.dart';
import 'package:retgoo_sdk/pages/login_page.dart';
import 'package:retgoo_sdk/protocol/api.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';
import 'package:retgoo_sdk/providers/notification_provider.dart';
import 'package:retgoo_sdk/providers/shortcut_provider.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';
import 'package:retgoo_sdk/widgets/retgoo_server.dart';

class RetGooApp extends StatelessWidget {
  final String title;
  final Map<String, UserTaskBuilder> userTasks;
  final String serverHost;
  final ThemeData theme;
  const RetGooApp(
      {Key key,
      this.title = 'RetGoo Portal',
      this.userTasks,
      this.serverHost,
      this.theme})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    registeredUserTasks = userTasks;
    baseURL = serverHost;

    return AnnotatedRegion(
      value: SystemUiOverlayStyle.dark.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: ChangeNotifierProvider(
        create: (context) => AuthenticationProvider(),
        child: MaterialApp(
          title: title,
          theme: theme ??
              ThemeData(
                primarySwatch: Colors.indigo,
                primaryColor: Colors.indigo.shade900,
                inputDecorationTheme: InputDecorationTheme(
                  border: OutlineInputBorder(),
                ),
                cardTheme: CardTheme(
                  clipBehavior: Clip.antiAlias,
                  elevation: 8.0,
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(8.0),
                  ),
                  shadowColor: Colors.black.withOpacity(0.5),
                ),
                visualDensity: VisualDensity.adaptivePlatformDensity,
              ),
          routes: {
            "/": (context) => HomePage(),
          },
          builder: (context, navigator) {
            return Selector<AuthenticationProvider, AuthenticationState>(
              selector: (context, provider) => provider.state,
              builder: (context, state, child) {
                if (state == AuthenticationState.initializing) {
                  return Scaffold(
                    body: Center(
                      child: CircularProgressIndicator(),
                    ),
                  );
                }

                if (state == AuthenticationState.unautenticated) {
                  return Navigator(
                    pages: [
                      MaterialPage(
                        child: LoginPage(),
                      ),
                    ],
                    onPopPage: (route, result) => route.didPop(result),
                  );
                }

                return MultiProvider(
                  providers: [
                    ChangeNotifierProvider(
                      create: (context) => ShortcutProvider(),
                    ),
                    ChangeNotifierProvider(
                      create: (context) => NotificationProvider(),
                    )
                  ],
                  child: RetGooServer(
                    child: child,
                  ),
                );
              },
              child: navigator,
            );
          },
          debugShowCheckedModeBanner: false,
        ),
      ),
    );
  }
}
