import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/protocol/api.dart';
import 'package:retgoo_sdk/providers/notification_provider.dart';
import 'package:websocket/websocket.dart';

class RetGooServer extends StatefulWidget {
  final Widget child;
  RetGooServer({Key key, this.child}) : super(key: key);

  static _RetGooServerState of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<_ServerX>().state;
  }

  @override
  _RetGooServerState createState() => _RetGooServerState();
}

class _RetGooServerState extends State<RetGooServer> {
  bool _isDisposed = false;
  WebSocket channel;

  @override
  Widget build(BuildContext context) {
    return _ServerX(
      state: this,
      child: widget.child,
    );
  }

  @override
  void initState() {
    super.initState();
    listen();
  }

  listen() async {
    channel =
        await WebSocket.connect("ws://$baseURL/protocol?token=$accessToken");

    final listener = (event) {
      final payload = json.decode(event);
      final messageId = payload["i"];

      try {
        final messageMethod = payload["m"];
        final dataString = payload["d"];
        final dataObject = json.decode(dataString);

        handleMethod(messageMethod, dataObject);
      } catch (e) {} finally {
        channel.add(json.encode({"i": messageId}));
      }
    };

    channel.stream.listen(listener).onDone(() {
      if (_isDisposed) {
        listen();
      }
    });
  }

  handleMethod(String method, dynamic data) {
    //print(method);

    switch (method) {
      case "notification_activity":
      case "process_instance_completed":
        Provider.of<NotificationProvider>(context, listen: false).refresh();
        break;

      default:
        if (_listeners.containsKey(method)) {
          _listeners[method](data);
        }
    }
  }

  @override
  void dispose() {
    _isDisposed = true;
    channel.close(0);
    super.dispose();
  }

  Map<String, ValueChanged<dynamic>> _listeners = {};
  setListener(String method, ValueChanged<dynamic> callback) {
    _listeners[method] = callback;
  }

  removeListener(String method) {
    _listeners.remove(method);
  }
}

class _ServerX extends InheritedWidget {
  final _RetGooServerState state;
  _ServerX({Key key, this.child, this.state}) : super(key: key, child: child);

  final Widget child;

  @override
  bool updateShouldNotify(_ServerX oldWidget) {
    return true;
  }
}
