import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/directform.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/models/variable.dart';
import 'package:retgoo_sdk/pages/webview_page.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';

typedef UserTask UserTaskBuilder();

abstract class UserTask {
  BuildContext context;
  DirectForm _directForm;
  final formKey = GlobalKey<FormState>();
  VariableValue variable(String name) {
    Variable v = _directForm?.variable?.findByName(name);
    if (v != null) {
      return _directForm?.variable?.valueOf(v.id);
    }

    return null;
  }

  Widget attach(BuildContext context) {
    _directForm =
        Provider.of<ProcessProvider>(context, listen: false).directForm;
    this.context = context;
    return Form(
      key: formKey,
      autovalidateMode: AutovalidateMode.always,
      child: build(context),
    );
  }

  Widget build(BuildContext context);
  Future<bool> executeFlowButton(String idControl) async {
    return Provider.of<ProcessProvider>(context, listen: false)
        .claimProcess(idControl);
  }

  void showWebviewHTML(String htmlText, {String title}) {
    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => WebviewPage(
          htmlText: htmlText,
        ),
      ),
    );
  }

  bool get valid {
    return formKey.currentState.validate();
  }

  ProcessInfo get processInfo => ProcessInfo(_directForm,
      Provider.of<AuthenticationProvider>(context, listen: false).currentUser);
}

class ProcessInfo {
  final DirectForm _directForm;
  final User _user;

  ProcessInfo(this._directForm, this._user);

  int get masterProcessId => _directForm.idMasterProcess;
  int get processInstanceId => _directForm.idProcessInstance;
  User get currentUser => _user;
}
